package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.CommonStringRequest;
import com.cultural.heritage.model.dto.photoCategory.PhotoCategoryAddRequest;
import com.cultural.heritage.model.dto.photoCategory.PhotoCategoryUpdateRequest;
import com.cultural.heritage.model.entity.BookingDate;
import com.cultural.heritage.model.entity.BookingTime;
import com.cultural.heritage.model.entity.PhotoCategory;
import com.cultural.heritage.model.entity.PhotoProducts;
import com.cultural.heritage.model.vo.photoCategory.PhotoCategoryVO;
import com.cultural.heritage.model.vo.photoProducts.PhotoProductsVO;
import com.cultural.heritage.service.book.BookingDateService;
import com.cultural.heritage.service.book.BookingTimeService;
import com.cultural.heritage.service.book.PhotoCategoryService;
import com.cultural.heritage.service.book.PhotoProductsService;
import com.cultural.heritage.service.common.CommonService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/photoCategory")
@Slf4j
@Tag(name = "写真类别管理模块")
public class PhotoCategoryController {


    @Resource
    private PhotoCategoryService photoCategoryService;


    @Resource
    private PhotoProductsService photoProductsService;


    @Resource
    private CommonService commonService;


    @Resource
    private BookingDateService bookingDateService;


    @Resource
    private BookingTimeService bookingTimeService;



    /**
     * Web端管理员添加写真类别
     * @param photoCategoryAddRequest  写真类别添加请求体
     * @return 写真类别id
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加写真类别", description = "参数：写真类别添加请求体，权限：管理员(admin, boss)，方法名：addPhotoCategory")
    public BaseResponse<Long> addPhotoCategory(@RequestBody PhotoCategoryAddRequest photoCategoryAddRequest) {
        if (photoCategoryAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        PhotoCategory photoCategory = new PhotoCategory();
        BeanUtils.copyProperties(photoCategoryAddRequest, photoCategory);
        // 校验
        photoCategoryService.validPhotoCategory(photoCategory, false);
        boolean save = photoCategoryService.save(photoCategory);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "写真类别添加失败");
        return ResultUtils.success(photoCategory.getId());
    }



    /**
     * Web端管理员更新写真类别
     * @param photoCategoryUpdateRequest   写真类别更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员更新写真类别", description = "参数：写真类别更新请求体，权限：管理员(admin, boss)，方法名：updatePhotoCategory")
    public BaseResponse<Boolean> updatePhotoCategory(@RequestBody PhotoCategoryUpdateRequest photoCategoryUpdateRequest) {
        if (photoCategoryUpdateRequest == null || photoCategoryUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        // 获取原有写真类别名称
        Long id = photoCategoryUpdateRequest.getId();
        PhotoCategory originPhotoCategory = photoCategoryService.getById(id);
        String originName = originPhotoCategory.getName();
        // 获取目标写真预约名称
        String targetName = photoCategoryUpdateRequest.getName();

        // 更新原有写真类别下所有的写真产品类名
        UpdateWrapper<PhotoProducts> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("categoryName", originName).set("categoryName", targetName);
        boolean update = photoProductsService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "写真产品类名更新失败");
        // 更新写真类别名称
        PhotoCategory photoCategory = new PhotoCategory();
        BeanUtils.copyProperties(photoCategoryUpdateRequest, photoCategory);
        // 校验
        photoCategoryService.validPhotoCategory(photoCategory, true);

        boolean result = photoCategoryService.updateById(photoCategory);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真类别名称更新失败");

        return ResultUtils.success(true);
    }


    /**
     * 删除写真类别
     * @param commonRequest  写真类别id
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员删除写真类别", description = "参数：写真类别id，权限：管理员(admin, boss)，方法名：deletePhotoCategory")
    public BaseResponse<Boolean> deletePhotoCategory(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        PhotoCategory photoCategory = photoCategoryService.getById(id);
        QueryWrapper<PhotoProducts> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("categoryName", photoCategory.getName());
        List<PhotoProducts> photoProductsList = photoProductsService.list(queryWrapper);
        // 获取预约日期列表
        List<BookingDate> bookingDateList = commonService.getItemsByIds(photoProductsList, bookingDateService, PhotoProducts::getId);
        // 获取预约时间列表
        List<BookingTime> bookingTimes = commonService.getItemsByIds(bookingDateList, bookingTimeService, BookingDate::getId);

        // 批量删除预约时间
        boolean removeTime = bookingTimeService.removeBatchByIds(bookingTimes);
        ThrowUtils.throwIf(!removeTime, ErrorCode.OPERATION_ERROR, "预约时间删除失败");

        // 批量删除预约日期
        boolean removeDate = bookingDateService.removeBatchByIds(bookingDateList);
        ThrowUtils.throwIf(!removeDate, ErrorCode.OPERATION_ERROR, "预约日期批量删除失败");

        // 删除该类别下所有写真产品
        if (!photoProductsList.isEmpty()) {
            boolean remove = photoProductsService.removeBatchByIds(photoProductsList);
            ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "写真产品批量删除失败");
        }
        // 删除写真类别
        boolean result = photoCategoryService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "写真类别删除失败");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员查询写真类别
     * @return 写真类别列表
     */
    @PostMapping("/list")
    @Operation(summary = "Web端管理员查询写真类别", description = "参数：无，权限：web端管理员，方法名：listCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<PhotoCategoryVO>> listCategory() {
        List<PhotoCategory> photoCategoryList = photoCategoryService.list();
        List<PhotoCategoryVO> photoCategoryVOS = photoCategoryList.stream().map(photoCategory -> {
            PhotoCategoryVO photoCategoryVO = new PhotoCategoryVO();
            BeanUtils.copyProperties(photoCategory, photoCategoryVO);
            return photoCategoryVO;
        }).toList();
        return ResultUtils.success(photoCategoryVOS);
    }



    /**
     * 小程序端用户查询写真类别
     * @return 写真类别列表
     */
    @PostMapping("/list/users")
    @Operation(summary = "小程序端用户查询写真类别", description = "参数：无，权限：所有人，方法名：listCategoryByUsers")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<PhotoCategoryVO>> listCategoryByUsers() {
        QueryWrapper<PhotoProducts> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("isShelves", 1);
        List<PhotoProducts> photoProductsList = photoProductsService.list(queryWrapper);
        List<String> typeNameList = photoProductsList.stream().map(PhotoProducts::getCategoryName).toList();

        QueryWrapper<PhotoCategory> categoryWrapper = new QueryWrapper<>();
        categoryWrapper.in("name", typeNameList);
        List<PhotoCategory> categoryList = photoCategoryService.list(categoryWrapper);

        List<PhotoCategoryVO> photoCategoryVOS = categoryList.stream().map(photoCategory -> {
            PhotoCategoryVO photoCategoryVO = new PhotoCategoryVO();
            BeanUtils.copyProperties(photoCategory, photoCategoryVO);
            return photoCategoryVO;
        }).toList();

        return ResultUtils.success(photoCategoryVOS);
    }






    /**
     * 小程序端用户根据类名查询写真产品
     * @param commonStringRequest 写真类名
     * @return 写真产品列表
     */
    @PostMapping("/list/id")
    @Operation(summary = "小程序端用户根据写真类别id查询写真产品", description = "参数：无，权限：所有人，方法名：listPhotoProductsByTypeName")
    public BaseResponse<List<PhotoProductsVO>> listPhotoProductsByTypeName(@RequestBody CommonStringRequest commonStringRequest) {
        if (commonStringRequest == null || StringUtils.isBlank(commonStringRequest.getType())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String categoryName = commonStringRequest.getType();
        QueryWrapper<PhotoProducts> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("categoryName", categoryName).eq("isShelves", 1);
        List<PhotoProducts> photoProductsList = photoProductsService.list(queryWrapper);

        List<PhotoProductsVO> photoProductsVOS = photoProductsList.stream().map(photoProducts -> {
            PhotoProductsVO photoProductsVO = new PhotoProductsVO();
            BeanUtils.copyProperties(photoProducts, photoProductsVO);
            return photoProductsVO;
        }).toList();

        return ResultUtils.success(photoProductsVOS);
    }


}
